﻿--[[

Battleground Raid Adjust (BGRA)

A simple add-on that allows the group leader to save a group configuration
outside fo the battleground, and restore that configuration once all of the
raid members have entered the Battleground.

Author: Ellric of Darkspear
Email: trbayne@gmail.com

]]


-- ==================================================================
-- Our Objects
-- ==================================================================

BGRA = {};
BGRA.SavedGroups = {};

-- ==================================================================
-- Properties
-- ==================================================================

BGRA.VERSION = 1.0;
BGRA.VERSIONSTRING = string.format("Version %.1f",BGRA.VERSION);
BGRA.DebugFlag = false;
BGRA.SavedGrouping = false;


-- ==================================================================
-- Constants
-- ==================================================================

BGRA.LOAD_CONFIRM_MSG = "|c0000BFFFBGRaidAjdust |c0000FF00" .. BGRA.VERSIONSTRING .. "|c0000BFFF is Loaded|r";
BGRA.VERSION_MSG = "|c0000BFFFBGRaidAjdust |c0000FF00" .. BGRA.VERSIONSTRING .. "|c0000BFFF|r";

-- ==================================================================
-- Command Line Hook Function
-- Note: using BGRaidAdjust:DoCmd type syntax doesn't work here
-- ==================================================================
function BGRA_SlashCommandHook(Cmd)

  BGRA.DoCmd(Cmd);
    
end

-- ==================================================================
-- Main Functions
-- ==================================================================

function BGRA.OnLoad()
    
    SLASH_BGRA1 = "/BGRA";
    SlashCmdList["BGRA"] = BGRA_SlashCommandHook;
    DEFAULT_CHAT_FRAME:AddMessage(BGRA.LOAD_CONFIRM_MSG);
    -- BGRA.Print(BGRA.LOAD_CONFIRM_MSG);
    
end


function BGRA.DoCmd(CommandLine)
    local i,j, cmd, param = string.find(CommandLine, "^([^ ]+)%s*(.*)$");
    if(not cmd) then cmd = command; end
    if(not cmd) then cmd = ""; end
    if(not param) then param = ""; end
    BGRA.Debug(string.format("BGRA parsed command = [%s]",cmd));
    if(cmd == "") then
        BGRA.DisplayUsage();
    else 
        cmd = string.upper(cmd);
        if (cmd == "VERSION") then
            BGRA.DisplayVersion();
        elseif (cmd == "SAVE") then
            BGRA.SaveCurrentGrouping();
        elseif (cmd == "SHOW") then
            BGRA.ShowCurrentGrouping();
        elseif (cmd == "RESET") then
            BGRA.ResetGrouping();
        elseif (cmd == "SHOWROSTER") then
            BGRA.ShowCurrentRoster();
        elseif (cmd == "DEBUG") then
            BGRA.ToggleDebug();
        elseif (cmd == "CLEAR") then
            BGRA.ClearTopSubGroups();
        elseif (cmd == "HELP") then
            BGRA.DisplayHelp();
        end     
        
    end   
end


function BGRA.ResetGrouping()
    
    local raidGroup, color, name;
	local CurrentRoster,rc,rs;
   
    if ( not IsRaidLeader() ) then
		BGRA.Print("BGRA: You must be the raid leader to complete this operation");
        return;
	end
    
    BGRA.Print("BGRA: Resetting Groups");
    if (BGRA.SavedGrouping == true) then
        
        CurrentRoster = BGRA.GetCurrentRoster();
        for name in pairs(CurrentRoster) do
            rc = CurrentRoster[name];
            rs = BGRA.SavedGroups[name];
            if (rs ~= nil) then
                if (rc.subgroup ~= rs.subgroup) then
                    SetRaidSubgroup(rc.index,rs.subgroup);        
                end
            end
        end
    end
end

function BGRA.SaveCurrentGrouping()
    
    BGRA.Print("BGRA Saving Group Configuration|r");
    
    -- Erase previous saved group
    BGRA.SavedGroups = {};
    BGRA.SavedGrouping = true;
    BGRA.SavedGroups = BGRA.GetCurrentRoster();
    BGRA.Print("BGRA Grouping Saved!|r");
end

function BGRA.ShowCurrentGrouping()
    
    local name, r, msg;
    BGRA.Print("================================|r");
    BGRA.Print("BGRA: Saved Group Config |r");
    BGRA.Print("================================|r");
    for name in pairs(BGRA.SavedGroups) do
        r = BGRA.SavedGroups[name];
        msg = string.format("[%d] %s|r",r.subgroup,name);
        BGRA.Print(msg);
    end
    BGRA.Print("================================|r");
    
end

function BGRA.GetCurrentRoster()
    
    local numRaidMembers = GetNumRaidMembers();
	local i, name, rank, subgroup, level, class, fileName, zone, online, isDead;
    local r;
    local lRoster;
    
    -- Erase previous saved group
    lRoster = {};
    for i=1, MAX_RAID_MEMBERS do
		if ( i <= numRaidMembers ) then
			name, rank, subgroup, level, class, fileName, zone, online, isDead = GetRaidRosterInfo(i);
            r = {};
            r.index = i;
            r.name = name;
            r.rank = rank;
            r.subgroup = subgroup;
            r.level = level;
            r.class = class;
            r.filename = fileName;
            r.zone = zone;
            r.online = online;
            r.isDead = isDead
            lRoster[name] = r;
        end
	end
    return lRoster;
end




function BGRA.ShowCurrentRoster()
    
    BGRA.Print("================================|r");
    BGRA.Print("BGRA: Current Roster |r");
    BGRA.Print("================================|r");
    local name, cr;
   
    cr = BGRA.GetCurrentRoster();
    
    for name in pairs(cr) do
        r = cr[name];
        msg = string.format("I:[%d] SG:[%d] %s|r",r.index,r.subgroup,name);
        BGRA.Print(msg);
	end
    BGRA.Print("================================|r");
end



function BGRA.ClearTopSubGroups()
    local numRaidMembers = GetNumRaidMembers();
    local i;
    if ( not IsRaidLeader() ) then
		BGRA.Print("BGRA: You must be the raid leader to complete this operation");
        return;
	end
    if (numRaidMembers <= 15) then
        for i=1, MAX_RAID_MEMBERS do
            if (i <= 5) then
                SetRaidSubgroup(i,5);
            elseif (i <= 10) then
                SetRaidSubgroup(i,6);
            elseif (i <= 15) then
                SetRaidSubgroup(i,7);
            end
        end
        --BGRA.Delay(.5);
    else
        BGRA.Print("BGRA: Too many raid members to complete this operation|r");
        return false
    end
end    

-- ==================================================================
-- Utility Functions
-- ==================================================================
    
function BGRA.DisplayUsage()
    BGRA.Print("BGRA Usage:");
    BGRA.Print("  |cffffffff/bgra help|r |cff2040ff [Displays Help]|r");
    BGRA.Print("  |cffffffff/bgra version|r |cff2040ff[Displays the BG Raid Adjust Version]|r");
    BGRA.Print("  |cffffffff/bgra save|r |cff2040ff [Saves your current group arrangement]|r");
    BGRA.Print("  |cffffffff/bgra show|r |cff2040ff [Displays the current saved grouping]|r");
    BGRA.Print("  |cffffffff/bgra clear|r |cff2040ff [Clears out the top 3 subgroups]|r");
    BGRA.Print("  |cffffffff/bgra reset|r |cff2040ff[Restores your saved grouping]|r");
    BGRA.Print("  |cffffffff/bgra debug|r |cff2040ff[Toggles display of debug messages]|r");
    BGRA.Print("  |cffffffff/bgra showroster|r |cff2040ff[Displays the current roster]|r");
end

function BGRA.DisplayHelp()
    BGRA.Print("|cff2040ffBGRA Basic Usage:|r");
    BGRA.Print("  |cffffffffSetup your groups outside of the Battlegrounds|r"
    BGRA.Print("  |cffffffffand save using |cff2040ff/bgra save|r");
    BGRA.Print("  |cffffffffOnce all your group memebers are inside|r")
    BGRA.Print("  |cffffffffthe Battleground, do a |cff2040ff/bgra clear|r");
    BGRA.Print("  |cffffffffto clear the top groups. |r");
    BGRA.Print("  |cffffffffNow do a |cff2040ff/bgra reset|r ");
    BGRA.Print("  |cffffffffto restore your previously saved grouping.|r");
    BGRA.Print("  |cff2040ffNote: ANY pugs or group members not present when you saved|r");
    BGRA.Print("  |cff2040ffNote: the grouping arrangement will be left at the bottom.|r");
end    

function BGRA.ToggleDebug()
    if (BGRA.DebugFlag == true) then
        BGRA.DebugFlag = false;
        BGRA.Print("BGRA: Debug Messages are now off|r");
    else
        BGRA.DebugFlag = true;
        BGRA.Print("BGRA: Debug Messages are now on|r");
    end
end

function BGRA.DisplayVersion()
    BGRA.Print(BGRA.VERSION_MSG);
end

function BGRA.Debug(msg)
    if (BGRA.DebugFlag == true) then
        local lmsg;
        lmsg = "BGRA DEBUG: " .. msg .. "|r";
        BGRA.Print(lmsg);
        
    end
end

-- Delay a given number of seconds
-- 1 = 1 second, .5 = 1/2 a second

function BGRA.Delay(length)
    
    local CurrentTime = GetTime();
    local i = CurrentTime;
    while ((i - CurrentTime) < length) do
        i = GetTime();
    end
end

function BGRA.SortByIndex(a,b) 
    
    return a.index < b.index; 
end

function BGRA.sortedpairs(t, comparator)

    local sortedKeys = {};
	table.foreach(t, function(k,v) table.insert(sortedKeys,k) end);
	table.sort(sortedKeys,comparator);
	local i = 0;
	local function _f(_s,_v)
		i = i + 1;
		local k = sortedKeys[i];
		if (k) then
			return k,t[k];
		end
	end
	return _f,nil,nil;
end

function BGRA.Print(msg,r,g,b,frame,id)
	if ( Print ) then
		Print(msg, r, g, b, frame, id);
		return;
	end

	if (not r) then r = 1.0; end
	if (not g) then g = 1.0; end
	if (not b) then b = 0.0; end
	if ( frame ) then
		frame:AddMessage(msg,r,g,b,id);
	else
		if ( DEFAULT_CHAT_FRAME ) then
			DEFAULT_CHAT_FRAME:AddMessage(msg, r, g, b,id);
		end
	end
end


